/**
 * 
 */
package org.processmining.models.visualcnet;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import javax.swing.SwingConstants;

import org.processmining.models.cnet.CNet;
import org.processmining.models.cnet.CNetNode;
import org.processmining.models.graphbased.AttributeMap;
import org.processmining.models.graphbased.directed.AbstractDirectedGraph;
import org.processmining.models.graphbased.directed.DirectedGraph;
import org.processmining.models.graphbased.directed.DirectedGraphEdge;
import org.processmining.models.graphbased.directed.DirectedGraphElement;
import org.processmining.models.graphbased.directed.DirectedGraphNode;

/**
 * Visualization of cnet
 * 
 * @author aadrians Oct 7, 2011
 * 
 */
public class CNetV extends AbstractDirectedGraph<CNetVNode, CNetVEdge> {
	private final Set<CNetVNode> nodes;
	private final Set<CNetVEdge> edges;

	protected CNetVNode createCNetVNode(AbstractDirectedGraph<CNetVNode, CNetVEdge> graph, CNetNode node, CNet ne) {
		return new CNetVNode(graph, node, ne);
	}

	protected CNetVNode createCNetVNode(CNetV graph, CNetVNode origNode) {
		return new CNetVNode(graph, origNode);
	}

	protected CNetVEdge createCNetVEdge(CNetVNode source, CNetVNode target) {
		return new CNetVEdge(source, target);
	}

	public CNetV() {
		nodes = new HashSet<CNetVNode>();
		edges = new HashSet<CNetVEdge>();

		getAttributeMap().put(AttributeMap.PREF_ORIENTATION, SwingConstants.WEST);
		getAttributeMap().put(AttributeMap.LABEL, "no label");
	}

	public CNetV(CNet cnet) {
		// add nodes
		Set<CNetNode> cnetNodes = cnet.getNodes();
		nodes = new HashSet<CNetVNode>(cnetNodes.size());

		Map<CNetNode, CNetVNode> map = new HashMap<CNetNode, CNetVNode>();
		for (CNetNode node : cnetNodes) {
			CNetVNode vNode = createCNetVNode(this, node, cnet);
			addNode(vNode);
			map.put(node, vNode);
		}

		// add edges
		edges = new HashSet<CNetVEdge>();
		Set<CNetNode> exploredCNet = new HashSet<CNetNode>(cnetNodes.size());
		for (CNetNode node : map.keySet()) {
			for (CNetNode succ : cnet.getSuccessors(node)) {
				if (!exploredCNet.contains(succ)) {
					CNetVNode source = map.get(node);
					CNetVNode target = map.get(succ);
					addArc(source, target, source.getLabel() + "-" + target.getLabel());
				}
			}

			for (CNetNode pred : cnet.getPredecessors(node)) {
				if (!exploredCNet.contains(pred)) {
					CNetVNode target = map.get(node);
					CNetVNode source = map.get(pred);
					addArc(source, target, source.getLabel() + "-" + target.getLabel());
				}
			}

			exploredCNet.add(node);
		}

		getAttributeMap().put(AttributeMap.PREF_ORIENTATION, SwingConstants.WEST);
		getAttributeMap().put(AttributeMap.LABEL, cnet.getLabel());
	}

	public Set<CNetVNode> getNodes() {
		return nodes;
	}

	public Set<CNetVEdge> getEdges() {
		return edges;
	}

	public void removeNode(DirectedGraphNode cell) {
		if (cell instanceof CNetVNode) {
			removeNode((CNetVNode) cell);
		}
	}

	public CNetVNode removeNode(CNetVNode flexNodeElement) {
		removeSurroundingEdges(flexNodeElement);
		return removeNodeFromCollection(nodes, flexNodeElement);
	}

	protected CNetV getEmptyClone() {
		return new CNetV();
	}

	protected Map<? extends DirectedGraphElement, ? extends DirectedGraphElement> cloneFrom(
			DirectedGraph<CNetVNode, CNetVEdge> graph) {
		HashMap<DirectedGraphElement, DirectedGraphElement> mapping = new HashMap<DirectedGraphElement, DirectedGraphElement>();

		for (CNetVNode a : graph.getNodes()) {
			CNetVNode newNode = createCNetVNode(this, a);
			this.addNode(newNode);
			mapping.put(a, newNode);
		}

		getAttributeMap().clear();
		AttributeMap map = graph.getAttributeMap();
		for (String key : map.keySet()) {
			getAttributeMap().put(key, map.get(key));
		}
		return mapping;
	}

	private void addNode(CNetVNode node) {
		nodes.add(node);
		graphElementAdded(node);
	}

	protected void checkAddEdge(CNetVNode source, CNetVNode target) {
		if (!nodes.contains(source) && !nodes.contains(target)) {
			throw new IllegalArgumentException("Cannot add an arc between " + source.toString() + " and "
					+ target.toString() + ", since one of these nodes is not in the graph.");
		}

	}

	public synchronized CNetVEdge addArc(CNetVNode source, CNetVNode target, String label) {
		checkAddEdge(source, target);
		CNetVEdge a = createCNetVEdge(source, target);
		if (edges.add(a)) {
			graphElementAdded(a);
			return a;
		} else {
			for (CNetVEdge existing : edges) {
				if (existing.equals(a)) {
					if (label != null) {
						existing.getAttributeMap().put(AttributeMap.LABEL, label);
					}
					return existing;
				}
			}
		}
		assert (false);
		return null;
	}

	public void removeEdge(@SuppressWarnings("rawtypes") DirectedGraphEdge edge) {
		if (edge instanceof CNetVEdge) {
			edges.remove(edge);
		} else {
			assert (false);
		}
		graphElementRemoved(edge);
	}

}
