package org.processmining.plugins.dcnet;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.font.LineBreakMeasurer;
import java.awt.font.TextAttribute;
import java.awt.font.TextLayout;
import java.awt.geom.Line2D;
import java.awt.geom.RoundRectangle2D;
import java.text.AttributedString;

public final class PopupDrawer {

	private static final String ELLIPSES = "...";

	private float maxPopupWidth = 300.0f;
	private float maxPopupHeight = 200.0f;
	private float popupPadding = 2.0f;
	private Font popupFont = new Font(Font.MONOSPACED, Font.PLAIN, 10);
	private Color popupBackgroundColor = Color.WHITE;
	private Color popupTextColor = Color.BLACK; // new Color(0, 0, 0, 130);

	public void drawLabelPopup(String label, Graphics2D g, float startX, float startY) {

		g.setFont(getPopupFont());

		// measure the text		
		float[] bounds = measureTextBounds(g, label);

		float strWidth = bounds[0];
		float strHeight = bounds[1];

		float labelStartX = startX + 15;

		g.setColor(getPopupBackgroundColor());
		float paddedStartX = labelStartX - (getPopupPadding() / 2);
		g.fill(new RoundRectangle2D.Float(paddedStartX, startY, strWidth + getPopupPadding(),
				strHeight + (getPopupPadding() / 2), getPopupPadding(), getPopupPadding()));
		g.setColor(Color.BLACK);
		g.setStroke(new BasicStroke(0.2f));
		g.draw(new RoundRectangle2D.Float(labelStartX - (getPopupPadding() / 2), startY, strWidth + getPopupPadding(),
				strHeight + (getPopupPadding() / 2), getPopupPadding(), getPopupPadding()));

		// Really draw the text
		g.setColor(getPopupTextColor());
		drawPopupText(g, label, labelStartX, startY);

		g.setColor(Color.GRAY);
		g.setStroke(new BasicStroke(0.25f));
		g.draw(new Line2D.Double(startX, startY, paddedStartX, startY + (strHeight / 2.0)));

	}

	private void drawPopupText(Graphics2D g, String text, float startX, float startY) {
		float labelX = startX;
		float labelY = startY;
		AttributedString attributedString = new AttributedString(text);
		attributedString.addAttribute(TextAttribute.FONT, g.getFont());
		attributedString.addAttribute(TextAttribute.FOREGROUND, g.getColor());
		LineBreakMeasurer breakMeasure = new LineBreakMeasurer(attributedString.getIterator(),
				g.getFontRenderContext());

		while (breakMeasure.getPosition() < text.length()) {

			int positionBefore = breakMeasure.getPosition();
			int positionAfter = breakMeasure.nextOffset(getMaxPopupWidth());

			TextLayout layout = breakMeasure.nextLayout(getMaxPopupWidth());
			float textHeight = layout.getDescent() + layout.getLeading() + layout.getAscent();

			if ((labelY + (4 * textHeight)) > getMaxPopupHeight()) {
				// Last Row				
				String shortenedLastRow = text.substring(positionBefore, Math.max(positionBefore, positionAfter - 3))
						.concat(ELLIPSES);
				AttributedString attrLastRow = new AttributedString(shortenedLastRow);
				attrLastRow.addAttribute(TextAttribute.FONT, g.getFont());
				attrLastRow.addAttribute(TextAttribute.FOREGROUND, g.getColor());
				TextLayout lastRowLayout = new TextLayout(attrLastRow.getIterator(), g.getFontRenderContext());
				float dx = lastRowLayout.isLeftToRight() ? 0 : (getMaxPopupWidth() - lastRowLayout.getAdvance());
				lastRowLayout.draw(g, labelX + dx, labelY + lastRowLayout.getAscent());
				break;
			}

			labelY += layout.getAscent();
			float dx = layout.isLeftToRight() ? 0 : (getMaxPopupWidth() - layout.getAdvance());
			layout.draw(g, labelX + dx, labelY);
			labelY += layout.getDescent() + layout.getLeading();
		}
	}

	private float[] measureTextBounds(Graphics2D g, String text) {
		AttributedString attributedText = new AttributedString(text);
		attributedText.addAttribute(TextAttribute.FONT, g.getFont());
		LineBreakMeasurer breakMeasure = new LineBreakMeasurer(attributedText.getIterator(), g.getFontRenderContext());

		float strHeight = 0;
		float strWidth = 0;

		while (breakMeasure.getPosition() < text.length()) {

			int positionBefore = breakMeasure.getPosition();
			int positionAfter = breakMeasure.nextOffset(getMaxPopupWidth());

			TextLayout layout = breakMeasure.nextLayout(getMaxPopupWidth());
			float textHeight = layout.getDescent() + layout.getLeading() + layout.getAscent();

			if ((strHeight + (4 * textHeight)) > getMaxPopupHeight()) {
				// Last Row				
				String shortenedLastRow = text.substring(positionBefore, Math.max(positionBefore, positionAfter - 3));
				AttributedString attrLastRow = new AttributedString(shortenedLastRow.concat(ELLIPSES));
				attrLastRow.addAttribute(TextAttribute.FONT, g.getFont());
				TextLayout lastRowLayout = new TextLayout(attrLastRow.getIterator(), g.getFontRenderContext());

				strWidth = Math.max(strWidth, lastRowLayout.getAdvance());
				strHeight += layout.getAscent() + layout.getDescent() + layout.getLeading();
				break;
			}

			strWidth = Math.max(strWidth, layout.getAdvance());
			strHeight += layout.getAscent() + layout.getDescent() + layout.getLeading();
		}
		return new float[] { strWidth, strHeight };
	}

	public float getMaxPopupWidth() {
		return maxPopupWidth;
	}

	public void setMaxPopupWidth(float maxPopupWidth) {
		this.maxPopupWidth = maxPopupWidth;
	}

	public float getMaxPopupHeight() {
		return maxPopupHeight;
	}

	public void setMaxPopupHeight(float maxPopupHeight) {
		this.maxPopupHeight = maxPopupHeight;
	}

	public float getPopupPadding() {
		return popupPadding;
	}

	public void setPopupPadding(float popupPadding) {
		this.popupPadding = popupPadding;
	}

	public Font getPopupFont() {
		return popupFont;
	}

	public void setPopupFont(Font popupFont) {
		this.popupFont = popupFont;
	}

	public Color getPopupBackgroundColor() {
		return popupBackgroundColor;
	}

	public void setPopupBackgroundColor(Color popupBackgroundColor) {
		this.popupBackgroundColor = popupBackgroundColor;
	}

	public Color getPopupTextColor() {
		return popupTextColor;
	}

	public void setPopupTextColor(Color popupTextColor) {
		this.popupTextColor = popupTextColor;
	}

}
